//
// Simple test of GLFW library

#include <GL/glew.h>
#include <GLFW/glfw3.h>
#include <stdlib.h>
#include <stdio.h>
#include	"Program.h"
#include	"VertexArray.h"
#include	"VertexBuffer.h"
#include	"glUtilities.h"
#include	"mat4.h"

#define	NUM_VERTICES	3
#define	VERTEX_SIZE		(3*sizeof(float))

static const float vertices [] = 
{
	-1.0f, -1.0f, 0.0f,
	 0.0f,  1.0f, 0.0f,
	 1.0f, -1.0f, 0.0f
};

Program			program;
VertexArray		vao;
VertexBuffer	buf;

void error ( int error, const char * description )
{
	fputs ( description, stderr );
}
void key ( GLFWwindow * window, int key, int scancode, int action, int mods )
{
	if ( key == GLFW_KEY_ESCAPE && action == GLFW_PRESS )
		glfwSetWindowShouldClose ( window, GL_TRUE );
}

void display ( GLFWwindow * window )
{
	glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	vao.bind ();	
	glDrawArrays ( GL_TRIANGLES, 0, NUM_VERTICES );		
	vao.unbind ();
}

void	reshape ( GLFWwindow * window, int w, int h )
{
	glViewport ( 0, 0, (GLsizei)w, (GLsizei)h );
}

int main ()
{
	glfwSetErrorCallback ( error );
	
	if ( !glfwInit() )
		exit ( 1 );
	
	glfwWindowHint ( GLFW_RESIZABLE, 1 );
	glfwWindowHint ( GLFW_DOUBLEBUFFER, 1 );
	glfwWindowHint ( GLFW_DEPTH_BITS, 24 );
	glfwWindowHint ( GLFW_CLIENT_API, GLFW_OPENGL_API );
	glfwWindowHint ( GLFW_CONTEXT_VERSION_MAJOR, 4 );
	glfwWindowHint ( GLFW_CONTEXT_VERSION_MINOR, 4 );
	glfwWindowHint ( GLFW_OPENGL_FORWARD_COMPAT, GL_TRUE );
	glfwWindowHint ( GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE );

	GLFWwindow * window = glfwCreateWindow ( 640, 480, "Simple example", NULL, NULL );
	
	if ( !window )
	{
		glfwTerminate ();
		exit ( 1 );
	}

	glfwMakeContextCurrent ( window );
	
    glewExperimental = GL_TRUE;
    glewInit   ();
    glGetError ();
	
	if ( !program.loadProgram ( "simple.glsl" ) )
	{
		printf ( "Error building program: %s\n", program.getLog ().c_str () );
		
		exit ( 2 );
	}
	
	program.bind ();
	vao.create   ();
	vao.bind     ();
	buf.create   ();
	buf.bind     ( GL_ARRAY_BUFFER );
	buf.setData  ( NUM_VERTICES * VERTEX_SIZE, vertices, GL_STATIC_DRAW );

	program.setAttrPtr ( "position", 3, VERTEX_SIZE, (void *) 0 );

	buf.unbind     ();
	vao.unbind     ();
	
	glfwMakeContextCurrent    ( window );
	glfwSwapInterval          ( 1 );
	glfwSetKeyCallback        ( window, key );
	glfwSetWindowSizeCallback ( window, reshape );
	
	while ( !glfwWindowShouldClose ( window ) )
	{
		display ( window );
		glfwSwapBuffers ( window );
		glfwPollEvents  ();
	}
	
	glfwDestroyWindow ( window );
	glfwTerminate     ();
	
	return 0;
}
